#include "..//stdafx.h"
#include "../Demo.h"
#include "Model.h"


void BuildModel()
{

 // CClockScene compScene;
	sgGetScene()->Clear();
	StaircaseModel hm;
	hm.Build(200, 800, 3000, 1, 24);

}

#define _USE_MATH_DEFINES
#include <math.h>

SG_VECTOR xAxe = {1, 0, 0};
SG_VECTOR yAxe = {0, 1, 0};
SG_VECTOR zAxe = {0, 0, 1};
SG_POINT zeroP = {0, 0, 0};

sgCContour*   StaircaseModel::BuildStepContour()
{
	sgCObject*   objcts[4];

	SG_POINT  aPnt = {m_in_radius, 0, 0};
	SG_POINT  bPnt = {m_out_radius, 0, 0};

	objcts[0] = sgCreateLine(aPnt.x, aPnt.y, aPnt.z, bPnt.x, bPnt.y, bPnt.z);
	
	sgFloat  angl = 2*M_PI/m_steps_count; 
	sgFloat  sinAng = sin(angl); 
	sgFloat  cosAngl = cos(angl); 
	SG_POINT  dPnt = {m_in_radius*cosAngl, m_in_radius*sinAng, 0};
	SG_POINT  cPnt = {m_out_radius*cosAngl, m_out_radius*sinAng, 0};

	SG_ARC   arc1;
	arc1.FromCenterBeginEnd(zeroP, bPnt, cPnt, false);

	objcts[1] = sgCreateArc(arc1);

	objcts[2] = sgCreateLine(cPnt.x, cPnt.y, cPnt.z, dPnt.x, dPnt.y, dPnt.z);

	SG_ARC   arc2;
	arc2.FromCenterBeginEnd(zeroP, dPnt, aPnt, false);

	objcts[3] = sgCreateArc(arc2);

	return  sgCContour::CreateContour(objcts, 4);

}

sgC3DObject*  StaircaseModel::BuildOneStep(const sgCContour* stepCont)
{
	SG_VECTOR extrVect = {0,0, m_stair_height/m_helix_count/m_steps_count/1.0f};
	return ((sgC3DObject*)sgKinematic::Extrude(*stepCont, NULL, 0, extrVect, true));
}

#define   GUIDE_WIDTH     50
#define   HANDRAIL_WIDTH    20

sgCContour*   StaircaseModel::BuildGuideContour(bool insideGuide)
{

	sgFloat   a1 = ((insideGuide)?m_in_radius:m_out_radius) - GUIDE_WIDTH/2;
	sgFloat   a2 = a1 + GUIDE_WIDTH/4;
	sgFloat   a3 = a1 + 3*GUIDE_WIDTH/4;
	sgFloat   a4 = a1 + GUIDE_WIDTH;

	sgFloat   b1 = -2*m_stair_height/m_helix_count/m_steps_count;
	sgFloat   b2 = 3*b1/4;
	sgFloat   b3 = -b2;
	sgFloat   b4 = -b1;

	sgCObject*   objcts[12];

	objcts[0] = sgCreateLine(a2, 0, b3,    a2, 0, b2);
	objcts[1] = sgCreateLine(a2, 0, b2,    a1, 0, b2);
	objcts[2] = sgCreateLine(a1, 0, b2,    a1, 0, b1);
	objcts[3] = sgCreateLine(a1, 0, b1,    a4, 0, b1);
	objcts[4] = sgCreateLine(a4, 0, b1,    a4, 0, b2);
	objcts[5] = sgCreateLine(a4, 0, b2,    a3, 0, b2);
	objcts[6] = sgCreateLine(a3, 0, b2,    a3, 0, b3);
	objcts[7] = sgCreateLine(a3, 0, b3,    a4, 0, b3);
	objcts[8] = sgCreateLine(a4, 0, b3,    a4, 0, b4);
	objcts[9] = sgCreateLine(a4, 0, b4,    a1, 0, b4);
	objcts[10] = sgCreateLine(a1, 0, b4,    a1, 0, b3);
	objcts[11] = sgCreateLine(a1, 0, b3,    a2, 0, b3);

	

	return  sgCContour::CreateContour(objcts, 12);
}

sgC3DObject*  StaircaseModel::BuildOneGuide(const sgCContour* guideCont,bool insideGuide)
{
	return (sgC3DObject*)sgKinematic::Spiral(*guideCont, NULL, 0, zeroP, zAxe, m_stair_height/m_helix_count, m_stair_height, 36, true);
}

sgCContour*   StaircaseModel::BuildBalusterContour()
{
	sgFloat hmin = 2*m_stair_height/m_helix_count/m_steps_count;
	sgFloat hmax = m_stair_height/m_helix_count/2;

	sgFloat xPos = m_out_radius - GUIDE_WIDTH/4;

	sgCObject*   objcts[3];

	SG_POINT   a1 = {xPos, 0, hmin+(hmax-hmin)/6};
	SG_POINT   a2 = {xPos-GUIDE_WIDTH/3, 0, hmin+(hmax-hmin)/2};
	SG_POINT   a3 = {xPos, 0, hmin+5*(hmax-hmin)/6};

	objcts[0] = sgCreateLine(xPos, 0, hmin,    a1.x, a1.y, a1.z);

	SG_ARC   arc;
	arc.FromThreePoints(a1, a3, a2, false);

	objcts[1] = sgCreateArc(arc);
	objcts[2] = sgCreateLine(a3.x, a3.y, a3.z,   xPos, 0, hmax);



	return  sgCContour::CreateContour(objcts, 3);
}

sgC3DObject*  StaircaseModel::BuildBaluster(const sgCContour* balusterCont)
{
	SG_POINT   axeP1 = {m_out_radius, 0, 0};
	SG_POINT   axeP2 = {m_out_radius, 0, 1};
	return (sgC3DObject*)sgKinematic::Rotation(*balusterCont, axeP1, axeP2, 360, false);
}

sgCContour*   StaircaseModel::BuildHandrailContour()
{
	sgFloat hmax = m_stair_height/m_helix_count/2;

	sgFloat   a1 = m_out_radius - GUIDE_WIDTH/2;
	sgFloat   a2 = a1 + GUIDE_WIDTH/4;
	sgFloat   a3 = a1 + 3*GUIDE_WIDTH/4;

	SG_POINT   pnt1 = {a2, 0, hmax};
	SG_POINT   pnt2 = {a2, 0, hmax+HANDRAIL_WIDTH};
	SG_POINT   pnt3 = {a3, 0, hmax+HANDRAIL_WIDTH};
	SG_POINT   pnt4 = {a3, 0, hmax};

	SG_ARC arc1;
	arc1.FromBeginEndNormalAngle(pnt1, pnt2, yAxe, 2*5*M_PI/6);
	SG_ARC arc2;
	arc2.FromBeginEndNormalAngle(pnt3, pnt4, yAxe, 2*5*M_PI/6);

	sgCObject*   objcts[4];
	objcts[0] = sgCreateArc(arc1);
	objcts[1] = sgCreateLine(pnt2.x, pnt2.y, pnt2.z, pnt3.x, pnt3.y, pnt3.z);
	objcts[2] = sgCreateArc(arc2);
	objcts[3] = sgCreateLine(pnt4.x, pnt4.y, pnt4.z, pnt1.x, pnt1.y, pnt1.z);

	return  sgCContour::CreateContour(objcts, 4);
}

sgC3DObject*  StaircaseModel::BuildHandrail(const sgCContour* HandrailCont)
{
	return (sgC3DObject*)sgKinematic::Spiral(*HandrailCont, NULL, 0, zeroP, zAxe, m_stair_height/m_helix_count, m_stair_height, 36, true);
}

void  StaircaseModel::Build(sgFloat in_rad, sgFloat out_rad, sgFloat stairH, unsigned int helix_cnt, unsigned int steps_cnt)
{
	sgC3DObject::AutoTriangulate(false, SG_DELAUNAY_TRIANGULATION, SG_SEPARATE_BUFFERS);
	 m_in_radius = in_rad;
	 m_out_radius = out_rad;
	 m_stair_height = stairH;
	 m_helix_count = helix_cnt;
	 m_steps_count = steps_cnt;

	 if (m_in_radius>m_out_radius || m_steps_count<1 || m_helix_count<1)
		 return;

	 sgCContour* cont = BuildStepContour();
	 sgC3DObject* firstStep = BuildOneStep(cont);
	 firstStep->Triangulate(SG_VERTEX_TRIANGULATION, SG_SEPARATE_BUFFERS);
	  firstStep->SetAttribute(SG_OA_COLOR, 20);
	 sgGetScene()->AttachObject(firstStep);
	 for (int i=1;i<steps_cnt*m_helix_count;i++)
	 {
		  sgC3DObject*  curStep = (sgC3DObject*)(firstStep->Clone());
		  SG_VECTOR shiftVec = {0,0, i*m_stair_height/m_helix_count/m_steps_count};
		  curStep->InitTempMatrix()->Translate(shiftVec);
		  curStep->GetTempMatrix()->Rotate(zeroP, zAxe, i*2*M_PI/steps_cnt);
		  curStep->ApplyTempMatrix();
		  curStep->DestroyTempMatrix();
		  curStep->Triangulate(SG_VERTEX_TRIANGULATION, SG_SEPARATE_BUFFERS);
		  curStep->SetAttribute(SG_OA_COLOR, 20);
		   sgGetScene()->AttachObject(curStep);
	 }

	 sgCContour* inGuideCont = BuildGuideContour(true);
	 sgCContour* outGuideCont = BuildGuideContour(false);
	 	
	 sgC3DObject* inGuide = BuildOneGuide(inGuideCont, true);
	 inGuide->Triangulate(SG_VERTEX_TRIANGULATION, SG_SEPARATE_BUFFERS);
	 sgGetScene()->AttachObject(inGuide);

	 sgC3DObject* outGuide = BuildOneGuide(outGuideCont, false);
	 outGuide->Triangulate(SG_VERTEX_TRIANGULATION, SG_SEPARATE_BUFFERS);
	 sgGetScene()->AttachObject(outGuide);

	  sgDeleteObject(inGuideCont);
	   sgDeleteObject(outGuideCont);

	   sgCContour* BalusterCont = BuildBalusterContour();
	   
	   sgC3DObject* firstBaluster = BuildBaluster(BalusterCont);
	   
	   for (int i=1;i<steps_cnt*m_helix_count;i++)
	   {
		   sgC3DObject*  curBaluster = (sgC3DObject*)(firstBaluster->Clone());
		   SG_VECTOR shiftVec = {0,0, i*m_stair_height/m_helix_count/m_steps_count};
		   curBaluster->InitTempMatrix()->Translate(shiftVec);
		   curBaluster->GetTempMatrix()->Rotate(zeroP, zAxe, i*2*M_PI/steps_cnt);
		   curBaluster->ApplyTempMatrix();
		   curBaluster->DestroyTempMatrix();
		   curBaluster->Triangulate(SG_VERTEX_TRIANGULATION, SG_SEPARATE_BUFFERS);
		   curBaluster->SetAttribute(SG_OA_COLOR, 20);
		   sgGetScene()->AttachObject(curBaluster);
	   }

	   SG_VECTOR shiftVec2 = {0,GUIDE_WIDTH/2, 0};
	   firstBaluster->InitTempMatrix()->Translate(shiftVec2);
	   firstBaluster->ApplyTempMatrix();
	   firstBaluster->DestroyTempMatrix();
	   firstBaluster->Triangulate(SG_VERTEX_TRIANGULATION, SG_SEPARATE_BUFFERS);
	   firstBaluster->SetAttribute(SG_OA_COLOR, 20);
	   sgGetScene()->AttachObject(firstBaluster);

	    sgDeleteObject(BalusterCont);
		
		sgCContour* HandrailCont = BuildHandrailContour();
		sgC3DObject* Handrail = BuildHandrail(HandrailCont);
		Handrail->Triangulate(SG_VERTEX_TRIANGULATION, SG_SEPARATE_BUFFERS);
		sgGetScene()->AttachObject(Handrail);

		sgDeleteObject(HandrailCont);
		

	 sgDeleteObject(cont);
}